"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.App = exports.DEFAULT_MEMORY = exports.DEFAULT_TIMEOUT_S = exports.DEFAULT_APP_NAME = exports.DEFAULT_PORT = void 0;
const body_parser_1 = require("body-parser");
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const functions = __importStar(require("firebase-functions"));
exports.DEFAULT_PORT = 5000;
exports.DEFAULT_APP_NAME = 'app';
exports.DEFAULT_TIMEOUT_S = 540;
exports.DEFAULT_MEMORY = '2GB';
class App {
    constructor(settings) {
        this.authMetadataMiddleware = [];
        this.route = (...routes) => {
            for (const route of routes) {
                route.setPreRequestMiddleware(...this.preRequestMiddleware);
                route.setPostRequestMiddleware(...this.postRequestMiddleware);
                route.setAuthMetadataMiddleware(...this.authMetadataMiddleware);
                this.app.use(route.expressRouter);
            }
        };
        this.subscribe = (job) => {
            return functions
                .runWith(job.settings)
                .pubsub.schedule(job.schedule)
                .onRun(job.handler);
        };
        this.onWrite = (eventTrigger) => {
            return functions
                .runWith(eventTrigger.settings)
                .firestore.document(eventTrigger.documentPath)
                .onWrite(eventTrigger.handler);
        };
        this.onUpdate = (eventTrigger) => {
            return functions
                .runWith(eventTrigger.settings)
                .firestore.document(eventTrigger.documentPath)
                .onUpdate(eventTrigger.handler);
        };
        this.onCreate = (eventTrigger) => {
            return functions
                .runWith(eventTrigger.settings)
                .firestore.document(eventTrigger.documentPath)
                .onCreate(eventTrigger.handler);
        };
        this.onDelete = (eventTrigger) => {
            return functions
                .runWith(eventTrigger.settings)
                .firestore.document(eventTrigger.documentPath)
                .onDelete(eventTrigger.handler);
        };
        this.start = () => {
            return functions
                .runWith({
                timeoutSeconds: this.timeoutSeconds,
                memory: this.memory,
                minInstances: this.minInstances,
                maxInstances: this.maxInstances,
            })
                .https.onRequest(this.app);
        };
        this.local = (port = exports.DEFAULT_PORT) => {
            return this.app.listen(port, () => console.log(`${exports.DEFAULT_APP_NAME} listening locally on ${port}`));
        };
        this.registerExpressMiddleware = (middleware) => {
            for (const m of middleware) {
                if (m.path) {
                    this.app.use(m.path, m.function);
                }
                else {
                    this.app.use(m.function);
                }
            }
        };
        const { name = exports.DEFAULT_APP_NAME, timeoutSeconds = exports.DEFAULT_TIMEOUT_S, memory = exports.DEFAULT_MEMORY, minInstances, maxInstances, expressMiddleware = [], preRequestMiddleware = [], postRequestMiddleware = [], authMetadataMiddleware = [], corsOptions = { origin: true }, } = settings;
        if (timeoutSeconds < 10 || timeoutSeconds > 540) {
            throw new Error('Invalid timeoutSec');
        }
        this.name = name;
        this.app = (0, express_1.default)();
        this.app.use((0, cors_1.default)(corsOptions));
        this.app.use((0, body_parser_1.json)());
        this.registerExpressMiddleware(expressMiddleware);
        this.timeoutSeconds = timeoutSeconds;
        this.memory = memory;
        this.minInstances = minInstances;
        this.maxInstances = maxInstances;
        this.preRequestMiddleware = preRequestMiddleware;
        this.postRequestMiddleware = postRequestMiddleware;
        this.authMetadataMiddleware = authMetadataMiddleware;
        // Health check
        this.app.get('/', (_, res) => res.status(200).send(this.name));
    }
    get expressApp() {
        return this.app;
    }
}
exports.App = App;
//# sourceMappingURL=app.js.map