"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Route = exports.USER_TOKEN_KEY = void 0;
const util_1 = require("@marsupialtail/util");
const express_1 = __importDefault(require("express"));
const response_1 = require("./response");
exports.USER_TOKEN_KEY = 'userToken';
class Route {
    constructor(baseEndpoint) {
        this.preRequestMiddleware = [];
        this.postRequestMiddleware = [];
        this.authMetadataMiddleware = [];
        this.secretAuthMiddleware = [];
        this.getRouter = () => this.router;
        this.getBaseEndpoint = () => this.baseEndpoint;
        this.function = (endpoint, options, callback) => {
            if (endpoint.length === 0 || endpoint[0] !== '/') {
                throw Error('Route endpoint must start with a slash');
            }
            this.router.post(endpoint, (req, res) => __awaiter(this, void 0, void 0, function* () {
                const startTimestamp = Date.now();
                const body = req.body;
                const cacheKey = this.getEndpointRequestKey(endpoint, req, options.cacheIgnoreProps);
                const onResponse = (code, body) => {
                    if (options.cacheMs) {
                        this.cache.set(cacheKey, {
                            code,
                            body,
                        }, options.cacheMs);
                    }
                };
                const response = new response_1.Response(body, res, onResponse);
                // Apply pre-request middleware
                for (const preRequestMiddleware of this.preRequestMiddleware) {
                    yield preRequestMiddleware(req, response);
                    if (response.isLocked) {
                        return;
                    }
                }
                try {
                    let metadata = {};
                    /**
                     * Attempt to fetch response from cache.
                     */
                    const cached = this.cache.get(cacheKey);
                    if (cached) {
                        return response.sendResponse(cached.code, cached.body, false);
                    }
                    /**
                     * Perform auth pre-check
                     */
                    for (const authMetadataMiddleware of this.authMetadataMiddleware) {
                        const authMetadata = yield authMetadataMiddleware(req, response, options.auth || false);
                        if (response.isLocked) {
                            break;
                        }
                        if (authMetadata) {
                            metadata = Object.assign(Object.assign({}, metadata), authMetadata);
                        }
                    }
                    for (const secretAuthMiddleware of this.secretAuthMiddleware) {
                        const secretAuthMetadata = yield secretAuthMiddleware(req, response, options.secretAuth || false);
                        if (response.isLocked) {
                            break;
                        }
                        if (secretAuthMetadata) {
                            metadata = Object.assign(Object.assign({}, metadata), secretAuthMetadata);
                        }
                    }
                    // Run the function if no response has been sent yet
                    if (!response.isLocked) {
                        try {
                            yield callback(body, response, metadata);
                        }
                        catch (e) {
                            console.error(e);
                            return response.internalServerError('Function error');
                        }
                    }
                    const endTimestamp = Date.now();
                    // Apply post-request middleware
                    for (const postRequestMiddleware of this.postRequestMiddleware) {
                        yield postRequestMiddleware(req, response, startTimestamp, endTimestamp);
                    }
                }
                catch (e) {
                    console.error(e);
                    return response.internalServerError('Endpoint error');
                }
            }));
        };
        this.setAuthMiddleware = (...middleware) => {
            this.authMetadataMiddleware = middleware;
        };
        this.setSecretMiddleware = (...middleware) => {
            this.secretAuthMiddleware = middleware;
        };
        this.setPreRequestMiddleware = (...middleware) => {
            this.preRequestMiddleware = middleware;
        };
        this.setPostRequestMiddleware = (...middleware) => {
            this.postRequestMiddleware = middleware;
        };
        this.getEndpointRequestKey = (endpoint, request, cacheIgnoreProps) => {
            const body = (0, util_1.copyWithKeysDeleted)(request.body, cacheIgnoreProps || []);
            return (endpoint +
                '=' +
                Object.keys(body)
                    .sort()
                    .map(k => `${k}:${body[k]}`)
                    .join(','));
        };
        if (baseEndpoint.length === 0 || baseEndpoint[0] !== '/') {
            throw Error('Route baseEndpoint must start with a slash');
        }
        this.baseEndpoint = baseEndpoint;
        this.router = express_1.default.Router();
        this.cache = new util_1.ExpiringCache();
    }
}
exports.Route = Route;
//# sourceMappingURL=route.js.map