import { ProtocolFunctionInterface } from '@marsupialtail/idl';
import express from 'express';
import { AuthMetadataMiddlewareFunction, PostRequestMiddlewareFunction, PreRequestMiddlewareFunction } from '../middleware';
import { ExpressRequest, ExpressResponse } from './express';
import { Response, ResponseBody } from './response';
export declare type FunctionCallback<RequestType, ResponseType, MetadataType> = (req: RequestType, res: Response<RequestType, ResponseType>, data: MetadataType) => Promise<void>;
export declare type CachedResponses = {
    [requestKey: string]: CachedResponse;
};
export declare type CachedResponse = {
    code: number;
    body?: ResponseBody;
};
export declare type ExpressRouteHandler = (req: ExpressRequest, res: ExpressResponse) => Promise<void>;
export declare class Route<MetadataType = void> {
    private router;
    private cache;
    private preRequestMiddleware;
    private postRequestMiddleware;
    private authMetadataMiddleware;
    constructor();
    get expressRouter(): express.Router;
    function: <RequestType, ResponseType_1>(protocol: ProtocolFunctionInterface<RequestType, ResponseType_1>, callback: FunctionCallback<RequestType, ResponseType_1, MetadataType>) => void;
    setAuthMetadataMiddleware: (...middleware: AuthMetadataMiddlewareFunction[]) => void;
    setPreRequestMiddleware: (...middleware: PreRequestMiddlewareFunction[]) => void;
    setPostRequestMiddleware: (...middleware: PostRequestMiddlewareFunction[]) => void;
    private getRequestKey;
    private callbackWithTimeout;
}
