"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Route = void 0;
const util_1 = require("@marsupialtail/util");
const express_1 = __importDefault(require("express"));
const response_1 = require("./response");
class Route {
    constructor() {
        this.preRequestMiddleware = [];
        this.postRequestMiddleware = [];
        this.authMetadataMiddleware = [];
        this.function = (protocol, callback) => {
            const options = protocol.options;
            if (options.path[0] !== '/') {
                throw Error('Protocol path must start with a slash');
            }
            // Get the Express routing function dynamically
            ;
            this.router[options.method.toLowerCase()](options.path, (req, res) => __awaiter(this, void 0, void 0, function* () {
                const startTimestamp = Date.now();
                const body = (options.method === 'GET' ? req.params : req.body);
                const requestKey = this.getRequestKey(req, options.cacheIgnoreProps);
                const onResponse = (code, body) => {
                    if (options.serverCacheMs) {
                        const requestCache = this.cache.get(options.cacheKey) || {};
                        requestCache[requestKey] = {
                            code,
                            body,
                        };
                        this.cache.set(options.cacheKey, requestCache, options.serverCacheMs);
                    }
                    if (options.invalidatesCacheKeys) {
                        this.cache.delete(...options.invalidatesCacheKeys);
                    }
                };
                const response = new response_1.Response(body, res, onResponse);
                // Apply pre-request middleware
                for (const preRequestMiddleware of this.preRequestMiddleware) {
                    yield preRequestMiddleware(req, response);
                    if (response.isLocked) {
                        return;
                    }
                }
                try {
                    let metadata = {};
                    /**
                     * Attempt to fetch response from cache.
                     */
                    const requestCache = this.cache.get(options.cacheKey) || {};
                    if ((0, util_1.hasKey)(requestCache, requestKey)) {
                        const cached = (0, util_1.getValue)(requestCache, requestKey);
                        return response.sendResponse(cached.code, cached.body, false);
                    }
                    /**
                     * Perform auth pre-check
                     */
                    for (const authMiddleware of this.authMetadataMiddleware) {
                        const authMetadata = yield authMiddleware(req, response, options.authRequired);
                        if (response.isLocked) {
                            break;
                        }
                        if (authMetadata) {
                            metadata = Object.assign(Object.assign({}, metadata), authMetadata);
                        }
                    }
                    // Run the function if no response has been sent yet
                    if (!response.isLocked) {
                        try {
                            yield this.callbackWithTimeout(callback, body, response, metadata, options.serverTimeoutMs);
                        }
                        catch (e) {
                            console.error(e);
                            return response.internalServerError('Function error');
                        }
                    }
                    const endTimestamp = Date.now();
                    // Apply post-request middleware
                    for (const postRequestMiddleware of this.postRequestMiddleware) {
                        yield postRequestMiddleware(req, response, startTimestamp, endTimestamp);
                    }
                }
                catch (e) {
                    console.error(e);
                    return response.internalServerError('Endpoint error');
                }
            }));
        };
        this.setAuthMetadataMiddleware = (...middleware) => {
            this.authMetadataMiddleware = middleware;
        };
        this.setPreRequestMiddleware = (...middleware) => {
            this.preRequestMiddleware = middleware;
        };
        this.setPostRequestMiddleware = (...middleware) => {
            this.postRequestMiddleware = middleware;
        };
        this.getRequestKey = (request, cacheIgnoreProps) => {
            const body = (0, util_1.copyWithKeysDeleted)(request.body, cacheIgnoreProps || []);
            return Object.keys(body)
                .sort()
                .map(k => `${k}:${body[k]}`)
                .join(',');
        };
        this.callbackWithTimeout = (callback, req, res, data, timeoutMs) => __awaiter(this, void 0, void 0, function* () {
            if (!timeoutMs) {
                return callback(req, res, data);
            }
            const handleCancelable = (0, util_1.cancelable)(callback(req, res, data));
            setTimeout(() => {
                handleCancelable.cancel();
                res.requestTimeout();
            }, timeoutMs);
            yield handleCancelable;
        });
        this.router = express_1.default.Router();
        this.cache = new util_1.ExpiringCache();
    }
    get expressRouter() {
        return this.router;
    }
}
exports.Route = Route;
//# sourceMappingURL=route.js.map